@@ -69,11 +69,17 @@ Use monthly Comtrade data:
 import warnings
 import pandas
 import numpy as np
+import pandas as pd
 import math
 from biotrade.faostat import faostat
 from biotrade.comtrade import comtrade
 from biotrade.common.products import comtrade_faostat_mapping
 
+ELEMENT_DICT = {
+    "element_code": [5610, 5622, 5910, 5922],
+    "element": ["import_quantity", "import_value", "export_quantity", "export_value"],
+}
+
 
 def replace_exclusively(df, code_column, code_dict, na_value=-1):
     """Replace codes with the dict and all other values with na_value
@@ -224,7 +230,7 @@ def transform_comtrade_using_faostat_codes(
             "unit",
             "element",
         ]
-        df_agg = df.groupby(index, dropna=False)["value"].agg(sum).reset_index()
+        df_agg = df.groupby(index, dropna=False)["value"].agg("sum").reset_index()
         # Check that the Comtrade data didn't change after aggregation
         assert math.isclose(df.value.sum(), df_agg.value.sum())
         df = df_agg
@@ -277,7 +283,7 @@ def merge_faostat_comtrade(
         >>> df_comtrade = transform_comtrade_using_faostat_codes(
         >>>     comtrade_table="monthly", faostat_code = [1632, 1633])
         >>> (df_comtrade.query("year >= year.max() -2")
-        >>>  .groupby(["reporter", "period"])["value"].agg(sum)
+        >>>  .groupby(["reporter", "period"])["value"].agg("sum")
         >>>  .reset_index()
         >>>  .value_counts(["reporter"])
         >>>  .reset_index().to_csv("/tmp/value_counts.csv")
@@ -293,12 +299,19 @@ def merge_faostat_comtrade(
     if faostat_code:
         df_faostat = faostat.db.select(faostat_table, product_code=faostat_code)
         product_names = df_faostat[["product_code", "product"]].drop_duplicates()
+        element_df = pd.DataFrame.from_dict(ELEMENT_DICT)
         # Convert trade values from 1000 USD to USD
-        selector = df_faostat["unit"] == "1000 US$"
+        codes = element_df[element_df["element"].str.endswith("_value")][
+            "element_code"
+        ].values.tolist()
+        selector = df_faostat["element_code"].isin(codes)
         df_faostat.loc[selector, "value"] = df_faostat.loc[selector, "value"] * 1e3
         df_faostat.loc[selector, "unit"] = "usd"
         # Convert tonnes to kg
-        selector = df_faostat["unit"] == "tonnes"
+        codes = element_df[element_df["element"].str.endswith("_quantity")][
+            "element_code"
+        ].values.tolist()
+        selector = df_faostat["element_code"].isin(codes)
         df_faostat.loc[selector, "value"] = df_faostat.loc[selector, "value"] * 1e3
         df_faostat.loc[selector, "unit"] = "kg"
     else:
@@ -340,7 +353,7 @@ def merge_faostat_comtrade(
             "unit",
             "element",
         ]
-        df_comtrade_agg = df_comtrade.groupby(index)["value"].agg(sum)
+        df_comtrade_agg = df_comtrade.groupby(index)["value"].agg("sum")
         # The last year is not necessarily complete and it might differ by
         # countries. For any country. Sum the values of the last 12 months instead.
         # We need to go back a bit further , because in March of 2022, there might
@@ -383,31 +396,37 @@ def merge_faostat_comtrade(
         # Add FAOSTAT product names
         df = df.merge(product_names, on="product_code")
     # Add faostat element codes to comtrade data for consistency
-    cols = ["element", "element_code", "unit"]
+    cols = ["element_code", "element"]
     element_code_faostat = pandas.DataFrame(columns=cols)
     db = faostat.db
     # Retrieve element codes of Faostat db
     for table_name in ["crop_trade", "forestry_trade"]:
         table = db.tables[table_name]
-        element_code = pandas.read_sql_query(
+        element_code = db.read_sql_query(
             table.select()
-            .distinct(table.c.element, table.c.element_code, table.c.unit)
-            .with_only_columns([table.c.element, table.c.element_code, table.c.unit]),
-            db.engine,
+            .distinct(table.c.element_code, table.c.element)
+            .with_only_columns(table.c.element_code, table.c.element)
         )
         element_code_faostat = pandas.concat(
             [element_code_faostat, element_code], ignore_index=True
         )
-    element_code_faostat = element_code_faostat.drop_duplicates(subset=cols)[
-        cols
-    ].reset_index(drop=True)
-    # Convert trade units from 1000 USD to USD
-    selector = element_code_faostat["unit"] == "1000 US$"
-    element_code_faostat.loc[selector, "unit"] = "usd"
-    # Convert tonnes to kg
-    selector = element_code_faostat["unit"] == "tonnes"
-    element_code_faostat.loc[selector, "unit"] = "kg"
-    df = df.merge(element_code_faostat, how="left", on=["element", "unit"])
+    element_code_faostat = element_code_faostat.drop_duplicates().reset_index(drop=True)
+    element_code_faostat = (
+        element_code_faostat[
+            element_code_faostat["element_code"].isin(ELEMENT_DICT["element_code"])
+        ].sort_values(by="element_code")
+    ).reset_index(drop=True)
+    # Put as int element code
+    element_code_faostat["element_code"] = element_code_faostat["element_code"].astype(
+        "int64"
+    )
+    # Put a check on element codes
+    if not element_code_faostat.equals(pd.DataFrame.from_dict(ELEMENT_DICT)):
+        warnings.warn(
+            f"Element types for trade have changed from\n{pd.DataFrame.from_dict(ELEMENT_DICT)}\nPlease check them"
+        )
+    # Merge on element
+    df = df.merge(element_code_faostat, how="left", on=["element"])
     # Fill nan with -1 to let potential joins on element_code column too for comtrade data
     df["element_code"] = df["element_code"].fillna(-1).astype(int)
     # 4. Concatenate FAOSTAT and Comtrade data
