@@ -46,6 +46,12 @@ of the monthly data shows that "quantity" is always empty:
     select * from raw_comtrade.monthly where quantity is not null limit 4;
     # Returns 0 rows
 
+The database structure is defined as SQLAlchemy objects, then later translated
+in either SQLite or PostGreSQL dialect. Show the database structure as defined
+in PostGreSQL:
+
+    pg_dump --schema-only -t raw_comtrade.yearly $BIOTRADE_DATABASE_URL > table_structure_dump.sql
+
 """
 # First party modules
 import logging
@@ -53,10 +59,9 @@ import pandas
 import re
 
 # Third party modules
-import comtradeapicall
-from sqlalchemy import Integer, Float, Text, UniqueConstraint
+from sqlalchemy import Integer, Float, Text, UniqueConstraint, DateTime, Boolean
 from sqlalchemy import Table, Column, MetaData, and_, or_
-from sqlalchemy import create_engine, inspect
+from sqlalchemy import create_engine, inspect, select
 from sqlalchemy.schema import CreateSchema
 
 # Internal modules
@@ -95,11 +100,20 @@ class DatabaseComtrade(Database):
             with self.engine.connect() as conn:
                 if not self.engine.dialect.has_schema(conn, self.schema):
                     conn.execute(CreateSchema(self.schema))
-                    conn.commit()
 
         # Describe table metadata and create them if they don't exist
         # Product table
         self.product = self.describe_product_table(name="product")
+        # Reporter/Partner table
+        self.reporter = self.describe_country_table(name="reporter")
+        self.partner = self.describe_country_table(name="partner")
+        # Additional info tables
+        self.quantity = self.describe_additional_info_table(name="quantity_unit")
+        self.flow = self.describe_additional_info_table(name="flow")
+        self.transport = self.describe_additional_info_table(
+            name="modality_of_transport"
+        )
+        self.custom = self.describe_additional_info_table(name="custom")
         # Trade tables with data at the HS 6 digit level
         self.monthly = self.describe_trade_table(name="monthly")
         self.yearly = self.describe_trade_table(name="yearly")
@@ -107,6 +121,12 @@ class DatabaseComtrade(Database):
         self.yearly_hs2 = self.describe_trade_table(name="yearly_hs2")
         self.tables = {
             "product": self.product,
+            "reporter": self.reporter,
+            "partner": self.partner,
+            "quantity_unit": self.quantity,
+            "flow": self.flow,
+            "modality_of_transport": self.transport,
+            "custom": self.custom,
             "monthly": self.monthly,
             "yearly": self.yearly,
             "yearly_hs2": self.yearly_hs2,
@@ -115,6 +135,49 @@ class DatabaseComtrade(Database):
         for table in self.tables.values():
             self.create_if_not_existing(table)
 
+    def describe_country_table(self, name):
+        """Define the metadata of a table containing reporter/partner codes
+
+        :param str table: name of the table to create
+        """
+        # Additional columns depending on the specific table
+        table_dict = {
+            "reporter": [
+                Column("reporter_code", Integer),
+                Column("reporter", Text),
+                Column("reporter_note", Text),
+                Column("reporter_iso2", Text),
+                Column("reporter_iso", Text),
+                UniqueConstraint(
+                    "reporter_code",
+                ),
+            ],
+            "partner": [
+                Column("partner_code", Integer),
+                Column("partner", Text),
+                Column("partner_note", Text),
+                Column("partner_iso2", Text),
+                Column("partner_iso", Text),
+                UniqueConstraint(
+                    "partner_code",
+                ),
+            ],
+        }
+        # Common structure of the table
+        table = Table(
+            name,
+            self.metadata,
+            Column("id", Integer),
+            Column("text", Text),
+            Column("entry_effective_date", DateTime),
+            Column("is_group", Boolean),
+            Column("entry_expired_date", DateTime),
+            schema=self.schema,
+        )
+        for col in table_dict[name]:
+            table.append_column(col)
+        return table
+
     def describe_product_table(self, name):
         """Define the metadata of a table containing product codes
 
@@ -135,6 +198,53 @@ class DatabaseComtrade(Database):
         )
         return table
 
+    def describe_additional_info_table(self, name):
+        """Define the metadata of tables containing additional info
+
+        :param str table: name of the table to create
+        """
+        # Additional columns depending on the specific table
+        table_dict = {
+            "quantity_unit": [
+                Column("unit_code", Integer),
+                Column("unit_abbreviation", Text),
+                Column("unit", Text),
+                UniqueConstraint(
+                    "unit_code",
+                ),
+            ],
+            "flow": [
+                Column("flow_code", Text),
+                Column("flow", Text),
+                UniqueConstraint(
+                    "flow_code",
+                ),
+            ],
+            "modality_of_transport": [
+                Column("mode_of_transport_code", Integer),
+                Column("mode_of_transport", Text),
+                UniqueConstraint(
+                    "mode_of_transport_code",
+                ),
+            ],
+            "custom": [
+                Column("custom_proc_code", Text),
+                Column("custom_proc", Text),
+                UniqueConstraint(
+                    "custom_proc_code",
+                ),
+            ],
+        }
+        # Common structure of the table
+        table = Table(
+            name,
+            self.metadata,
+            schema=self.schema,
+        )
+        for col in table_dict[name]:
+            table.append_column(col)
+        return table
+
     def describe_trade_table(self, name):
         """Define the metadata of a table containing Comtrade data.
 
@@ -173,7 +283,7 @@ class DatabaseComtrade(Database):
             Column("reporter_code", Integer, index=True),
             Column("partner_code", Integer, index=True),
             Column("partner_2_code", Integer),
-            Column("customs_proc_code", Text),
+            Column("custom_proc_code", Text),
             Column("mode_of_supply_code", Text),
             Column("mode_of_transport_code", Integer),
             Column("product_code", Text, index=True),
@@ -298,7 +408,8 @@ class DatabaseComtrade(Database):
             )
         )
         # Execute delete statement
-        stmt.execute()
+        with self.engine.connect() as conn:
+            conn.execute(stmt)
         self.logger.info(
             "Delete data from database table %s, from %s to %s",
             table,
@@ -321,7 +432,7 @@ class DatabaseComtrade(Database):
         period_end=None,
     ):
         """
-        Select comtrade trade data for the given arguments
+        Select comtrade trade data for the given arguments, performing a join with additional information tables
 
         :param str table: name of the database table to select from
         :param list or str reporter: list of reporter names
@@ -342,8 +453,11 @@ class DatabaseComtrade(Database):
             >>> from biotrade.comtrade import comtrade
             >>> comtrade.db.select("monthly", product_code="440791")
 
+        Select yearly time series of saw bean oil crude and other
+
+            >>> comtrade.db.select("yearly", product_code=["150710", "150790"])
+
         """
-        table = self.tables[table]
         # Change character or integer arguments to lists suitable for a
         # column.in_() clause or for a list comprehension.
         if isinstance(reporter, str):
@@ -360,79 +474,37 @@ class DatabaseComtrade(Database):
             product_code_start = [product_code_start]
         if isinstance(flow, str):
             flow = [flow]
-        # Build the select statement
-        stmt = table.select()
-        # Define complementary data and columns to be selected and renamed
-        column_dict = {
-            "reporterCode": "reporter_code",
-            "reporterDesc": "reporter",
-            "reporterCodeIsoAlpha3": "reporter_iso",
-            "PartnerCode": "partner_code",
-            "PartnerDesc": "partner",
-            "PartnerCodeIsoAlpha3": "partner_iso",
-            "qtyCode": "unit_code",
-            "qtyDescription": "unit",
-        }
-        # reporter data
-        reporter_data = comtradeapicall.getReference("reporter")[
-            ["reporterCode", "reporterDesc", "reporterCodeIsoAlpha3"]
-        ].rename(columns=column_dict)
-        # partner data
-        partner_data = comtradeapicall.getReference("partner")[
-            ["PartnerCode", "PartnerDesc", "PartnerCodeIsoAlpha3"]
-        ].rename(columns=column_dict)
-        # quantity data
-        quantity_data = comtradeapicall.getReference("qtyunit")[
-            ["qtyCode", "qtyDescription"]
-        ].rename(columns=column_dict)
-        # define a compiled regex
-        regex_pat = re.compile(r"\W+")
-        # flow data
-        flow_data = comtradeapicall.getReference("flow")[["id", "text"]].rename(
-            columns={"id": "flow_code", "text": "flow"}
-        )
-        # rename flow column content to snake case using the compiled regex
-        flow_data["flow"] = (
-            flow_data["flow"].str.replace(regex_pat, "_", regex=True).str.lower()
-        )
-        # mode of transport data
-        mot_data = (
-            comtradeapicall.getReference("mot")[["id", "text"]]
-            .astype({"id": int})
-            .rename(
-                columns={
-                    "id": "mode_of_transport_code",
-                    "text": "mode_of_transport",
-                }
-            )
-        )
-        # custom procedure data
-        customs_data = comtradeapicall.getReference("customs")[["id", "text"]].rename(
-            columns={
-                "id": "customs_proc_code",
-                "text": "customs",
-            }
-        )
+        # Select complementary tables
+        reporter_table = self.tables["reporter"]
+        partner_table = self.tables["partner"]
+        quantity_table = self.tables["quantity_unit"]
+        flow_table = self.tables["flow"]
+        transport_table = self.tables["modality_of_transport"]
+        custom_table = self.tables["custom"]
         if reporter is not None:
-            # Obtain codes from reporter data
-            reporter = reporter_data[
-                reporter_data.reporter.isin(reporter)
-            ].reporter_code.tolist()
-            # Add already defined reporter codes
-            if isinstance(reporter_code, list):
-                reporter_code += reporter
-            else:
-                reporter_code = reporter
+            # Obtain codes from reporter table
+            stmt = reporter_table.select().where(
+                reporter_table.c.reporter.in_(reporter)
+            )
+            # Define the new select table
+            reporter_table = stmt.subquery()
         if partner is not None:
-            # Obtain codes from partner data
-            partner = partner_data[
-                partner_data.partner.isin(partner)
-            ].partner_code.tolist()
-            # Add already defined reporter codes
-            if isinstance(partner_code, list):
-                partner_code += partner
-            else:
-                partner_code = partner
+            # Obtain codes from partner table
+            stmt = partner_table.select().where(partner_table.c.partner.in_(partner))
+            # Define the new select table
+            partner_table = stmt.subquery()
+        if flow is not None:
+            # Define a compiled regex
+            regex_pat = re.compile(r"\W+")
+            # Rename flow list to snake case using the compiled regex
+            flow = [regex_pat.sub("_", item).lower() for item in flow]
+            # Obtain codes from flow table
+            stmt = flow_table.select().where(flow_table.c.flow.in_(flow))
+            # Define the new select table
+            flow_table = stmt.subquery()
+        # Build the main select statement
+        table = self.tables[table]
+        stmt = table.select()
         if reporter_code is not None:
             stmt = stmt.where(table.c.reporter_code.in_(reporter_code))
         if partner_code is not None:
@@ -441,29 +513,121 @@ class DatabaseComtrade(Database):
             stmt = stmt.where(table.c.product_code.in_(product_code))
         if product_code_start is not None:
             stmt = stmt.where(
-                or_(table.c.product_code.ilike(f"{c}%") for c in product_code_start)
+                or_(
+                    table.c.product_code.ilike(f"{code}%")
+                    for code in product_code_start
+                )
+            )
+        if period_start is not None:
+            stmt = stmt.where(table.c.period >= period_start)
+        if period_end is not None:
+            stmt = stmt.where(table.c.period <= period_end)
+        # Subquery results
+        table = stmt.subquery()
+        # Join tables
+        join_table = select(
+            table,
+            reporter_table.c.reporter,
+            reporter_table.c.reporter_iso,
+            partner_table.c.partner,
+            partner_table.c.partner_iso,
+            quantity_table.c.unit,
+            flow_table.c.flow,
+            transport_table.c.mode_of_transport,
+            custom_table.c.custom_proc,
+        ).select_from(
+            table.join(
+                reporter_table,
+                table.c.reporter_code == reporter_table.c.reporter_code,
+            )
+            .join(
+                partner_table,
+                table.c.partner_code == partner_table.c.partner_code,
+            )
+            .join(
+                quantity_table,
+                table.c.unit_code == quantity_table.c.unit_code,
+            )
+            .join(
+                flow_table,
+                table.c.flow_code == flow_table.c.flow_code,
+            )
+            .join(
+                transport_table,
+                table.c.mode_of_transport_code
+                == transport_table.c.mode_of_transport_code,
+            )
+            .join(
+                custom_table,
+                table.c.custom_proc_code == custom_table.c.custom_proc_code,
+            )
+        )
+        # Query the database and return a data frame
+        df = self.read_sql_query(join_table)
+        return df
+
+    def select_simple(
+        self,
+        table,
+        reporter_code=None,
+        partner_code=None,
+        product_code=None,
+        product_code_start=None,
+        period_start=None,
+        period_end=None,
+    ):
+        """
+        Select comtrade trade data for the given arguments
+
+        :param str table: name of the database table to select from
+        :param list or int reporter_code: list of reporter codes
+        :param list or int partner_code: list of partner codes
+        :param list or int product_code: list of product codes
+            Exact matches of product codes
+        :param list or int product_code_start: list of product codes
+            Partial matches of all products that start with this code
+        :param int period_start: filter on start period data
+        :param int period_end: filter on end period data
+        :return: A data frame of trade flows
+
+        For example select monthly time series of Oak sawnwood trade
+
+            >>> from biotrade.comtrade import comtrade
+            >>> comtrade.db.select_simple("monthly", product_code="440791")
+
+        """
+        # Change character or integer arguments to lists suitable for a
+        # column.in_() clause or for a list comprehension.
+        if isinstance(reporter_code, int):
+            reporter_code = [reporter_code]
+        if isinstance(partner_code, int):
+            partner_code = [partner_code]
+        if isinstance(product_code, str):
+            product_code = [product_code]
+        if isinstance(product_code_start, str):
+            product_code_start = [product_code_start]
+        # Build the main select statement
+        table = self.tables[table]
+        stmt = table.select()
+        if reporter_code is not None:
+            stmt = stmt.where(table.c.reporter_code.in_(reporter_code))
+        if partner_code is not None:
+            stmt = stmt.where(table.c.partner_code.in_(partner_code))
+        if product_code is not None:
+            stmt = stmt.where(table.c.product_code.in_(product_code))
+        if product_code_start is not None:
+            stmt = stmt.where(
+                or_(
+                    table.c.product_code.ilike(f"{code}%")
+                    for code in product_code_start
+                )
             )
-        if flow is not None:
-            # Rename flow list to snake case using the compiled regex
-            flow = [regex_pat.sub("_", item).lower() for item in flow]
-            # Obtain codes from flow data
-            flow_code = flow_data[flow_data.flow.isin(flow)].flow_code.tolist()
-            stmt = stmt.where(table.c.flow_code.in_(flow_code))
         if period_start is not None:
             stmt = stmt.where(table.c.period >= period_start)
         if period_end is not None:
             stmt = stmt.where(table.c.period <= period_end)
         # Query the database and return a data frame
         df = self.read_sql_query(stmt)
-        # Merge with complementary data
-        df = (
-            df.merge(reporter_data, on="reporter_code", how="left")
-            .merge(partner_data, on="partner_code", how="left")
-            .merge(quantity_data, on="unit_code", how="left")
-            .merge(flow_data, on="flow_code", how="left")
-            .merge(mot_data, on="mode_of_transport_code", how="left")
-            .merge(customs_data, on="customs_proc_code", how="left")
-        )
         return df
 
     def select_long(self, *args, **kwargs):
