@@ -29,6 +29,7 @@ Where the cronjob_update_db.py file contains
 """
 # Built-in modules
 from pathlib import Path
+import warnings
 import tempfile
 import shutil
 from zipfile import ZipFile
@@ -37,6 +38,7 @@ import pytz
 import os
 import re
 import time
+from collections import OrderedDict
 
 try:
     import requests
@@ -58,8 +60,10 @@ except Exception as e:
     msg += "but you can still use other methods.\n"
     print(msg, str(e))
 
+
 # Internal modules
 from biotrade.common.url_request_header import HEADER
+from biotrade.comtrade.pump_products import PumpProducts
 
 
 class Pump:
@@ -131,6 +135,13 @@ class Pump:
             self.token = os.environ["COMTRADE_TOKEN"]
         # Path of CSV log file storing API parameters and download status
         self.csv_log_path = self.parent.data_dir / "pump_comtrade_api_args.csv"
+        # Maximum number of rows in the Comtrade Free API limit
+        self.max_row_free_api_limit = 1e5
+
+    @property
+    def products(self):
+        """Download, cached and read trade data grouped by products"""
+        return PumpProducts(self)
 
     def sanitize_variable_names(self, df, renaming_from, renaming_to):
         """
@@ -138,26 +149,20 @@ class Pump:
         Use snake case in column names and replace some symbols
         """
         # Rename columns to snake case
-        df.rename(
-            columns=lambda x: re.sub(r" ", "_", str(x)).lower(), inplace=True
-        )
+        df.rename(columns=lambda x: re.sub(r" ", "_", str(x)).lower(), inplace=True)
         # Remove parenthesis and dots, used only for human readable dataset
         df.rename(columns=lambda x: re.sub(r"[()\.]", "", x), inplace=True)
         # Replace $ sign by d, used only for human readable dataset
         df.rename(columns=lambda x: re.sub(r"\$", "d", x), inplace=True)
         # Rename columns using the naming convention defined in self.column_names
-        mapping = self.column_names.set_index(renaming_from).to_dict()[
-            renaming_to
-        ]
+        mapping = self.column_names.set_index(renaming_from).to_dict()[renaming_to]
         # Discard nan keys of mapping dictionary
         mapping.pop(np.nan, None)
         df.rename(columns=mapping, inplace=True)
         # Rename column content to snake case using a compiled regex
         regex_pat = re.compile(r"\W+")
         if "flow" in df.columns:
-            df["flow"] = (
-                df["flow"].str.replace(regex_pat, "_", regex=True).str.lower()
-            )
+            df["flow"] = df["flow"].str.replace(regex_pat, "_", regex=True).str.lower()
             # Remove the plural "s"
             df["flow"] = df["flow"].str.replace("s", "", regex=True)
         return df
@@ -190,13 +195,13 @@ class Pump:
         )
         # Variable for checking successful gz download
         download_successful = False
-        # Default sleep time before downloading gz files
-        sleep_time = 5
+        # Default time to download gz files
+        time_start = time.time()
+        sleep_time = 0
         # Send the request: if successful put the gz files into the temporary
-        # folder else retry and double the wait time at each try
+        # folder else retry.
         # If the wait raises to more than 1 hour the loop stops.
         while not download_successful and sleep_time < 3600:
-            time.sleep(sleep_time)
             try:
                 # Create a request
                 comtradeapicall.bulkDownloadFinalFile(
@@ -213,12 +218,9 @@ class Pump:
                 # No data available
                 if len(os.listdir(temp_dir)) == 0:
                     response_code = 404
-            except Exception as error:
+            except (Exception, BaseException) as error:
                 response_code = error
-            # To avoid system exit exception
-            except BaseException as base_error:
-                response_code = base_error
-            sleep_time *= 2
+            sleep_time = time.time() - time_start
             self.logger.info(f"HTTP response code: {response_code}")
         return temp_dir, response_code
 
@@ -306,32 +308,61 @@ class Pump:
         """
         # List of chunk rows
         chunk_list = []
-        # Preallocate a dataframe
-        df = pandas.DataFrame()
         # Read the gz files
         for temp_file in os.listdir(temp_dir):
-            # Loop on gz files and upload them
-            try:
-                df = pandas.read_csv(
-                    temp_dir / temp_file,
-                    sep="\t",
-                    dtype={"datasetCode": str, "cmdCode": str, "mosCode": str},
-                )
-            # Continue with the next file
-            except Exception as error_gz:
+            # Preallocate a dataframe
+            df = pandas.DataFrame()
+            nr_attempts = 0
+            df_from_gz = False
+            while not df_from_gz and nr_attempts < 10:
+                # Loop on gz files and upload them
+                try:
+                    df = pandas.read_csv(
+                        temp_dir / temp_file,
+                        sep="\t",
+                        dtype={
+                            "datasetCode": str,
+                            "cmdCode": str,
+                            "mosCode": str,
+                        },
+                    )
+                    df_from_gz = True
+                # Continue until max number of attempts is reached
+                except Exception as error_gz:
+                    self.logger.info(
+                        f"Unable to load data from {temp_file} for period {api_period} due to\n {error_gz}.\n Retrying to download it.."
+                    )
+                    try:
+                        file_str = re.split(r"[-\[\]]", temp_file)[2]
+                        frequency = file_str[1]
+                        reporter_code = int(file_str[2:5])
+                        period = int(file_str[5:-2])
+                        comtradeapicall.bulkDownloadFinalFile(
+                            self.token,
+                            temp_dir,
+                            typeCode="C",
+                            freqCode=frequency,
+                            clCode="HS",
+                            period=period,
+                            reporterCode=reporter_code,
+                            decompress=False,
+                        )
+                        response_code = 200
+                    except (Exception, BaseException) as error:
+                        response_code = error
+                    self.logger.info(f"Response: {response_code}")
+                    nr_attempts += 1
+            if nr_attempts == 10:
                 self.logger.warning(
-                    f"Unable to load data from {temp_file} for period {api_period} due to\n {error_gz}"
+                    f"Reached the max number of attempts, {nr_attempts}. Unable to load data from {temp_file} for period {api_period}"
                 )
-                continue
             # If length is > 0 select rows
             if not df.empty:
                 # Remove potential white spaces between tab delimiter in string columns
                 strip_cols = df.select_dtypes(["object"]).columns
                 df[strip_cols] = df[strip_cols].apply(lambda x: x.str.strip())
                 nrow_before = len(df)
-                memory_before = round(
-                    df.memory_usage(deep=True).sum() / (1024**2), 2
-                )
+                memory_before = round(df.memory_usage(deep=True).sum() / (1024**2), 2)
                 selector = (
                     (df["customsCode"] == "C00")
                     & (df["motCode"] == 0)
@@ -351,9 +382,7 @@ class Pump:
                 elif table_name == "yearly_hs2":
                     # Store codes with bioeconomy label and 2 digits, not differentiating modality of transport (0), supply ("0"), 2nd partner (0)  and procedures ("C00")
                     # only (re)exported and (re)imported data
-                    selector_2d = selector & df["cmdCode"].isin(
-                        bioeconomy_tuple
-                    )
+                    selector_2d = selector & df["cmdCode"].isin(bioeconomy_tuple)
                     df = df[selector_2d]
                 msg = f"Before filtering {temp_file[:-3]} file, the dataframe occupies {memory_before} MB and number of rows are {nrow_before}, after {round(df.memory_usage(deep=True).sum() / (1024**2), 2)} MB with {len(df)} rows"
                 print(msg)
@@ -443,9 +472,7 @@ class Pump:
                         elif table_name == "yearly_hs2":
                             # Store codes with bioeconomy label and 2 digits
                             df_chunk = df_chunk[
-                                df_chunk["Commodity Code"].isin(
-                                    bioeconomy_tuple
-                                )
+                                df_chunk["Commodity Code"].isin(bioeconomy_tuple)
                             ]
                         elif table_name == "yearly":
                             # Store codes with bioeconomy label and 6 digits
@@ -566,9 +593,7 @@ class Pump:
                 "12",
             ]
         # Date object of today
-        current_date = datetime.datetime.now(
-            pytz.timezone("Europe/Rome")
-        ).date()
+        current_date = datetime.datetime.now(pytz.timezone("Europe/Rome")).date()
         # Loop on year and eventually month, depending on the frequency
         # parameter
         for period in period_block:
@@ -578,10 +603,7 @@ class Pump:
             for month in month_list:
                 if frequency == "M":
                     # Data not available in the future
-                    if (
-                        datetime.datetime(period, int(month), 1).date()
-                        > current_date
-                    ):
+                    if datetime.datetime(period, int(month), 1).date() > current_date:
                         break
                 # Construct the period to pass to transfer_csv_chunk method
                 api_period = int(str(period) + month)
@@ -643,7 +665,7 @@ class Pump:
         :param (string) table_name, name of Comtrade db table
         :param (string) frequency, "M" for monthly data or "A" for annual
         :param (int) start_year, year to start the download from, defaults to
-            2016 if not specified
+        last year if not specified
 
         Return an error if some periods are not uploaded to the database.
 
@@ -653,16 +675,15 @@ class Pump:
             >>> comtrade.pump.update_db(table_name = "monthly", frequency = "M")
 
         """
-        if start_year is None:
-            start_year = 2016
         # Adjust frequency parameter in case it is wrongly provided
         if table_name in ("yearly", "yearly_hs2"):
             frequency = "A"
         elif table_name == "monthly":
             frequency = "M"
-        current_year = (
-            datetime.datetime.now(pytz.timezone("Europe/Rome")).date().year
-        )
+        current_year = datetime.datetime.now(pytz.timezone("Europe/Rome")).date().year
+        # If start year not specified, update from the last year only
+        if start_year is None:
+            start_year = current_year - 1
         # Check if data from the start year are present into the database
         data_present = self.db.check_data_presence(
             table_name,
@@ -670,9 +691,6 @@ class Pump:
             current_year,
             frequency,
         )
-        # If data are already inside DB, update from the year before last only
-        if data_present:
-            start_year = current_year - 1
         # Transfer from api bulk requests to db
         self.transfer_bulk_csv(
             table_name,
@@ -698,6 +716,8 @@ class Pump:
     ):
         """Download a CSV file from the UN Comtrade data API and return a pandas data frame.
 
+        TODO: make arguments optional.
+
         The data API is documented at https://comtrade.un.org/data/doc/api/
 
         Argument names are the same as the Comtrade API argument names.
@@ -740,9 +760,7 @@ class Pump:
 
         # Load the data in json format
         if fmt == "json":
-            response = requests.get(
-                url=url_api_call, headers=self.header, stream=True
-            )
+            response = requests.get(url=url_api_call, headers=self.header, stream=True)
             print(f"HTTP response code: {response.status_code}")
             json_content = json.load(response.raw)
             # If the data was downloaded incorrectly, raise an error with the validation status
@@ -801,28 +819,111 @@ class Pump:
             >>> comtrade.pump.update_db_parameter()
 
         """
-        # Reload the data from Comtrade
-        hs = comtradeapicall.getReference("cmd:HS")
-        hs = hs.rename(
-            columns={
-                "id": "product_code",
-                "text": "product_description",
-                "isLeaf": "is_leaf",
-                "aggrLevel": "aggregate_level",
-            }
-        )
-        duplicated = hs.duplicated("product_code")
-        if any(duplicated):
-            self.logger.info(
-                "Dropping the following duplicated rows:\n %s", hs[duplicated]
+        # Dict for mapping table names and columns
+        table_name_dict = {
+            "cmd:HS": "product",
+            "reporter": "reporter",
+            "partner": "partner",
+            "qtyunit": "quantity_unit",
+            "flow": "flow",
+            "mot": "modality_of_transport",
+            "customs": "custom",
+        }
+        # First element of the ordered dict is the unique constraint column of the related table
+        table_col_dict = {
+            "cmd:HS": OrderedDict(
+                {
+                    "id": "product_code",
+                    "text": "product_description",
+                    "isLeaf": "is_leaf",
+                    "aggrLevel": "aggregate_level",
+                }
+            ),
+            "reporter": OrderedDict(
+                {
+                    "reporterCode": "reporter_code",
+                    "reporterDesc": "reporter",
+                    "reporterNote": "reporter_note",
+                    "reporterCodeIsoAlpha2": "reporter_iso2",
+                    "reporterCodeIsoAlpha3": "reporter_iso",
+                    "entryEffectiveDate": "entry_effective_date",
+                    "isGroup": "is_group",
+                    "entryExpiredDate": "entry_expired_date",
+                }
+            ),
+            "partner": OrderedDict(
+                {
+                    "PartnerCode": "partner_code",
+                    "PartnerDesc": "partner",
+                    "partnerNote": "partner_note",
+                    "PartnerCodeIsoAlpha2": "partner_iso2",
+                    "PartnerCodeIsoAlpha3": "partner_iso",
+                    "entryEffectiveDate": "entry_effective_date",
+                    "isGroup": "is_group",
+                    "entryExpiredDate": "entry_expired_date",
+                }
+            ),
+            "qtyunit": OrderedDict(
+                {
+                    "qtyCode": "unit_code",
+                    "qtyAbbr": "unit_abbreviation",
+                    "qtyDescription": "unit",
+                }
+            ),
+            "flow": OrderedDict({"id": "flow_code", "text": "flow"}),
+            "mot": OrderedDict(
+                {
+                    "id": "mode_of_transport_code",
+                    "text": "mode_of_transport",
+                }
+            ),
+            "customs": OrderedDict(
+                {
+                    "id": "custom_proc_code",
+                    "text": "custom_proc",
+                }
+            ),
+        }
+        # Load data of complementary Comtrade tables
+        for table in table_col_dict.keys():
+            df = comtradeapicall.getReference(table)
+            df = df.rename(columns=table_col_dict[table])
+            if "flow" in df.columns:
+                # Rename column content to snake case using a compiled regex
+                regex_pat = re.compile(r"\W+")
+                df["flow"] = (
+                    df["flow"].str.replace(regex_pat, "_", regex=True).str.lower()
+                )
+            # Delete existing data in the database
+            self.logger.info("Dropping existing %s table.", table_name_dict[table])
+            # session.execute(delete_stmt)
+            with self.db.engine.connect() as conn:
+                conn.execute(self.db.tables[table_name_dict[table]].delete())
+            # Remove potential duplicated id codes of the new data
+            duplicated = df.duplicated(list(table_col_dict[table].items())[0][1])
+            if any(duplicated):
+                self.logger.info(
+                    "Dropping the following duplicated rows from %s new data:\n %s",
+                    table_name_dict[table],
+                    df[duplicated],
+                )
+                df = df[~duplicated]
+            # Keep only columns actually present in the database structure
+            cols_in_db = self.db.tables[table_name_dict[table]].columns.keys()
+            # Warn about missing columns
+            missing_cols = set(df.columns) - set(cols_in_db)
+            if missing_cols:
+                msg = "The following columns are present in the Comtrade metadata "
+                msg += f"{missing_cols} but they are not present in the database "
+                msg += f"{table_name_dict[table]} table structure\n {cols_in_db} \n "
+                msg += f"or in the {table_name_dict[table]} renaming dict\n"
+                msg += f"{table_col_dict[table]}"
+                warnings.warn(msg)
+            # Store the data in the database
+            self.db.append(
+                df[cols_in_db], table_name_dict[table], drop_description=False
             )
-            hs = hs[~duplicated]
-        # Delete existing data in the database
-        self.logger.info("Dropping existing product table rows.")
-        self.db.tables["product"].delete().execute()
-        # Store the data in the database
-        self.db.append(hs, "product", drop_description=False)
-        self.logger.info("The product table has been updated.")
+            self.logger.info("The %s table has been updated.", table_name_dict[table])
 
     def write_log(
         self,
@@ -934,9 +1035,9 @@ class Pump:
         # list of years
         years = [str(i) for i in range(start_year, end_year + 1)]
         for reporter_code in reporters.id:
-            reporter_name = reporters.text[
-                reporters.id == reporter_code
-            ].to_string(index=False)
+            reporter_name = reporters.text[reporters.id == reporter_code].to_string(
+                index=False
+            )
             # https://comtrade.un.org/data/doc/api
             # "1 request every second (per IP address or authenticated user)."
             time.sleep(2)
@@ -952,18 +1053,14 @@ class Pump:
                 nr_product = 5
             # Select the first 5 products code at a time to avoid row limits to download
             product_block = product_code[
-                product_counter
-                * nr_product : (product_counter + 1)
-                * nr_product
+                product_counter * nr_product : (product_counter + 1) * nr_product
             ]
             # no more products to query
             while product_block != []:
                 # differentiate selection of the period based on the frequency
                 if frequency == "A":
                     # selection of 5 years
-                    period = years[
-                        period_counter * 5 : (period_counter + 1) * 5
-                    ]
+                    period = years[period_counter * 5 : (period_counter + 1) * 5]
                 elif frequency == "M":
                     # selection of 1 year(12 months)
                     period = years[period_counter : period_counter + 1]
@@ -1056,9 +1153,7 @@ class Pump:
                 )
                 # Trace API parameters and db status into table
                 self.write_log(
-                    timedate=datetime.datetime.now(
-                        pytz.timezone("Europe/Rome")
-                    ),
+                    timedate=datetime.datetime.now(pytz.timezone("Europe/Rome")),
                     table=table_name,
                     max=100000,
                     type="C",
@@ -1106,9 +1201,9 @@ class Pump:
         years = [str(year - i) for i in range(1, 6)]
         years = ",".join(years)
         for reporter_code in reporters.id:
-            reporter_name = reporters.text[
-                reporters.id == reporter_code
-            ].to_string(index=False)
+            reporter_name = reporters.text[reporters.id == reporter_code].to_string(
+                index=False
+            )
             download_successful = False
             # https://comtrade.un.org/data/doc/api
             # "1 request every second (per IP address or authenticated user)."
