@@ -10,7 +10,35 @@ Licenced under the MIT licence
 JRC biomass Project.
 Unit D1 Bioeconomy.
 
-Usage: download FAOSTAT datasets and store them in the database defined in
+Download FAOSTAT datasets.
+
+## Renaming variables
+
+It is important to be able to use variables in a sound way. Meaningful variable
+names in snake case makes comparing and merging data across different sources
+easier. In practice renaming is performed on column names and also on the content
+of some columns, such as the element or the product column.
+
+- The function `choose_column_renaming` decides which columns will be renamed
+  according to `config_data/column_names.csv`
+
+- The method `faostat.sanitize_variable_names` renames columns according to the
+  names defined in the mapping table.
+
+
+## Download FAOSTAT data into a data frame
+
+Simply download a compressed CSV file from FAOSTAT and read it into a data
+frame:
+
+    >>> zip_file_name = faostat.pump.datasets["forestry_production"]
+    >>> faostat.pump.download_zip_csv(zip_file_name)
+    >>> fp = faostat.pump.read_df("forestry_production")
+
+
+## Update your database
+
+Usage example: download FAOSTAT datasets and store them in the database defined in
 `faostat.db`:
 
     >>> from biotrade.faostat import faostat
@@ -39,7 +67,6 @@ except Exception as e:
     print(msg, str(e))
 
 # Third party modules
-import logging
 import pandas
 import numpy as np
 
@@ -101,16 +128,32 @@ class Pump:
     data frame for a specific country or a specific product, using the
     faostat.db.select method.
 
-    Update a dataset by downloading it again from FAOSTAT:
+    Update a dataset by downloading it again from FAOSTAT,
+    reading it, then writing it to the database. Note this uses
+    low level function, all steps can be handled by the update
+    method automatically:
 
-        >>> faostat.pump.download_zip_csv("Forestry_E_All_Data_(Normalized).zip")
+        >>> zip_file_name = faostat.pump.datasets["forestry_production"]
+        >>> faostat.pump.download_zip_csv(zip_file_name)
         >>> fp = faostat.pump.read_df("forestry_production")
         >>> faostat.self.db.write_df(fp, "forestry_production")
 
+    Which methods calls which one?
+
+    - `update` calls:
+        - `download_zip_csv`
+        - `transfer_to_db`
+    - `transfer_to_db` calls `transfer_csv_to_db_in_chunks`
+    - `transfer_csv_to_db_in_chunks` calls `sanitize_variable_names`
+
+    Other methods:
+    - `read_df` uses read_zip_csv_to_df.
+    - `read_df` is not used by the `update` and `transfer_to_db` methods.
+      `read_df` should be kept for small datasets that are read directly into a
+      data frame in memory.
+
     """
 
-    # Log debug and error messages
-    logger = logging.getLogger("biotrade.faostat")
     # Define URL request headers
     header = HEADER
     # Base URL to load data from the website
@@ -139,14 +182,14 @@ class Pump:
     }
 
     def __init__(self, parent):
-        # Default attributes #
         self.parent = parent
+        self.logger = self.parent.logger
         self.db = self.parent.db
         self.data_dir = self.parent.data_dir
         # Mapping table used to rename columns
         self.column_names = self.parent.column_names
         # Number of lines to transfer from csv files to the database at once
-        self.chunk_size = 10**5
+        self.chunk_size = 10**7
 
     def download_zip_csv(self, zip_file_name):
         """Download a compressed csv file from the FAOSTAT website
@@ -202,9 +245,10 @@ class Pump:
         return df
 
     def sanitize_variable_names(self, df, column_renaming, short_name):
-        # TODO: use the function sanitize_variable_names common/sanitise.py
         """Sanitize column names using the mapping table.
-        Use snake case in product and element names"""
+        Use snake case in product and element names
+        # TODO: use the function sanitize_variable_names common/sanitise.py
+        """
         # Rename columns to snake case
         df.rename(columns=lambda x: re.sub(r"\W+", "_", str(x)).lower(), inplace=True)
         # Columns of the db table
@@ -251,10 +295,10 @@ class Pump:
         # ft.flag.unique()
         # array(['', '*', 'R', 'Cv', 'P', 'A'], dtype=object)
         if "flag" in df.columns:
-            df.flag.fillna("", inplace=True)
+            df["flag"] = df["flag"].fillna("")
         return df
 
-    def read_df(self, short_name):
+    def read_df(self, short_name, reload=False):
         """Read an entire zip csv file to a data frame
 
         Not recommended for large datasets which don't fit into memory.
@@ -268,9 +312,18 @@ class Pump:
         >>> fl = faostat.pump.read_df("forest_land")
 
         """
-        # Read the compressed CSV into a data frame
+        zip_file_name = self.datasets[short_name]
+        zip_file = self.data_dir / zip_file_name
+        # Download the zip file is if it is not already in biotrade_data
+        if not zip_file.exists():
+            msg = f"\nThe file '{zip_file_name}' is absent from '{self.data_dir}'."
+            self.logger.info(msg)
+        if reload or not zip_file.exists():
+            self.logger.info(f"Downloading {zip_file_name} from FAOSTAT.")
+            self.download_zip_csv(zip_file_name)
+        # Read the file into a data frame
         df = self.read_zip_csv_to_df(
-            zip_file=self.data_dir / self.datasets[short_name],
+            zip_file=zip_file,
             column_renaming=choose_column_renaming(short_name),
             short_name=short_name,
         )
@@ -294,7 +347,14 @@ class Pump:
                 csv_file_name = temp_dir / re.sub(
                     ".zip$", ".csv", self.datasets[short_name]
                 )
-                encoding_var = "latin1"
+                if short_name in [
+                    "forestry_trade",
+                    "land_cover",
+                    "land_use",
+                ]:
+                    encoding_var = "latin1"
+                else:
+                    encoding_var = "utf-8"
                 # Test if the file is corrupted
                 with open(csv_file_name, "r", encoding=encoding_var) as csvfile:
                     # Detect the delimiter
