@@ -17,7 +17,7 @@ def test_put_mirror_beside():
     df = pandas.DataFrame(
         {
             "reporter_code": [1, 2, 2],
-            "partner_code": [2, 3, 1],
+            "partner_code": [2, 1, 3],
             "value": [10, 20, 30],
             "element": ["export", "import", "import"],
         }
@@ -26,10 +26,10 @@ def test_put_mirror_beside():
     expected = pandas.DataFrame(
         {
             "reporter_code": [1, 2, 2, 3],
-            "partner_code": [2, 3, 1, 2],
+            "partner_code": [2, 1, 3, 2],
             "value": [10, 20, 30, np.nan],
             "element": ["export", "import", "import", "export"],
-            "value_mirror": [30, np.nan, 10, 20],
+            "value_mirror": [20, 10, np.nan, 30],
         }
     )
     pandas.testing.assert_frame_equal(output, expected)
